==> server-core-3.5.1-sources.jar: com/xebialabs/deployit/task/deployment/UpgradeDeploymentTask.java <==
package com.xebialabs.deployit.task.deployment;

import com.xebialabs.deployit.plugin.PojoConverter;
import com.xebialabs.deployit.plugin.api.deployment.execution.DeploymentStep;
import com.xebialabs.deployit.plugin.api.deployment.specification.Delta;
import com.xebialabs.deployit.plugin.api.deployment.specification.DeltaSpecification;
import com.xebialabs.deployit.plugin.api.udm.DeployedApplication;
import com.xebialabs.deployit.repository.ChangeSet;
import com.xebialabs.deployit.repository.RepositoryObjectEntity;
import com.xebialabs.deployit.repository.RepositoryService;
import com.xebialabs.deployit.security.SecurityCallback;
import com.xebialabs.deployit.security.SecurityTemplate;
import com.xebialabs.deployit.security.UsernameAndPasswordCredentials;
import com.xebialabs.deployit.task.DeploymentTask;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import java.util.List;

import static com.google.common.collect.Lists.newArrayList;

@SuppressWarnings("serial")
public class UpgradeDeploymentTask extends DeploymentTask {

    private DeltaSpecification specification;
    private DeployedApplication oldApp;
    private transient PojoConverter pojoConverter;

    public UpgradeDeploymentTask(final DeltaSpecification specification, final DeployedApplication oldApp, List<? extends DeploymentStep> steps, RepositoryService repositoryService, PojoConverter pojoConverter, PojoConverter.Context... pojoConverterContexts) {
		super(steps, repositoryService, pojoConverterContexts);
        this.specification = specification;
        this.oldApp = oldApp;
        this.pojoConverter = pojoConverter;
        initFromDeployment(specification.getDeployedApplication());
    }

    @Override
    public void performPreFlightChecks() {
        UsernameAndPasswordCredentials owner = getOwnerCredentials();
		SecurityTemplate.executeAs(owner.getUsername(), owner.getPassword(), new SecurityCallback<Object>() {
            @Override
            public Object doAs() {
                PojoConverter.Context context = pojoConverter.getContext();
                try {
                    repositoryService.checkReferentialIntegrity(determineRepositoryChanges(context));
                } finally {
                    context.destroy();
                }

                return null;
            }
        });
    }

    private ChangeSet determineRepositoryChanges(PojoConverter.Context context) {
        final DeployedApplication newDeployment = specification.getDeployedApplication();

        // Save the mappings that are stored as subnodes of the deployment that was just created
        final List<RepositoryObjectEntity> createRoes = newArrayList();
        final List<RepositoryObjectEntity> updateRoes = newArrayList();
        final List<String> deleteRoeIds = newArrayList();

        for (Delta operation : specification.getDeltas()) {
            switch (operation.getOperation()) {
                case CREATE:
                    createRoes.add(context.toEntity(operation.getDeployed()));
                    newDeployment.addDeployed(operation.getDeployed());
                    logger.info("The CI {} was {} in the repository", operation.getDeployed().getId(), operation.getOperation());
                    break;
                case DESTROY:
                    oldApp.getDeployeds().remove(operation.getPrevious());
                    updateRoes.add(context.toEntity(oldApp));
                    deleteRoeIds.add(operation.getPrevious().getId());
                    logger.info("The CI {} was {} in the repository", operation.getPrevious().getId(), operation.getOperation());
                    break;
                case MODIFY:
	            case NOOP:
                    updateRoes.add(context.toEntity(operation.getDeployed()));
                    newDeployment.addDeployed(operation.getDeployed());
                    logger.info("The CI {} was {} in the repository", operation.getDeployed().getId(), operation.getOperation());
                    break;
            }
        }

        // Update the deployment to include the creating task id and to refer to the mappings that were just created
        RepositoryObjectEntity updatedNewDeploymentEntity = context.toEntity(newDeployment);
        updatedNewDeploymentEntity.setCreatingTaskId(getId());
        updateRoes.add(updatedNewDeploymentEntity);

        ChangeSet changeset = new ChangeSet();
        changeset.setDeleteEntityIds(deleteRoeIds);
        changeset.setCreateEntities(createRoes);
        changeset.setUpdateEntities(updateRoes);

        return changeset;
    }

	@Override
	public void doAfterTaskStateChangedToDone() {
		UsernameAndPasswordCredentials owner = getOwnerCredentials();
		SecurityTemplate.executeAs(owner.getUsername(), owner.getPassword(), new SecurityCallback<Object>() {
			@Override
			public Object doAs() {
                PojoConverter.Context context = pojoConverter.getContext();
                try {
                    repositoryService.execute(determineRepositoryChanges(context));
                } finally {
                    context.destroy();
                }

				return null;
			}
		});
	}

    private static final Logger logger = LoggerFactory.getLogger(UpgradeDeploymentTask.class);

    @Override
    public DeploymentType getDeploymentType() {
        return DeploymentTask.DeploymentType.UPGRADE;
    }
}

==> server-core-3.6.3-sources.jar: com/xebialabs/deployit/task/deployment/UpgradeDeploymentTask.java <==
package com.xebialabs.deployit.task.deployment;

import com.xebialabs.deployit.plugin.api.deployment.execution.DeploymentStep;
import com.xebialabs.deployit.plugin.api.deployment.specification.Delta;
import com.xebialabs.deployit.plugin.api.deployment.specification.DeltaSpecification;
import com.xebialabs.deployit.plugin.api.udm.ConfigurationItem;
import com.xebialabs.deployit.plugin.api.udm.DeployedApplication;
import com.xebialabs.deployit.repository.ChangeSet;
import com.xebialabs.deployit.repository.RepositoryService;
import com.xebialabs.deployit.repository.WorkDir;
import com.xebialabs.deployit.security.SecurityCallback;
import com.xebialabs.deployit.security.SecurityTemplate;
import com.xebialabs.deployit.security.UsernameAndPasswordCredentials;
import com.xebialabs.deployit.task.DeploymentTask;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import java.util.List;

import static com.google.common.collect.Lists.newArrayList;

@SuppressWarnings("serial")
public class UpgradeDeploymentTask extends DeploymentTask {

    private DeltaSpecification specification;
    private DeployedApplication oldApp;

    public UpgradeDeploymentTask(final DeltaSpecification specification, final DeployedApplication oldApp, List<? extends DeploymentStep> steps, RepositoryService repositoryService, WorkDir oldWorkDir, WorkDir newWorkDir) {
		super(steps, repositoryService, oldWorkDir, newWorkDir);
        this.specification = specification;
        this.oldApp = oldApp;
        initFromDeployment(specification.getDeployedApplication());
    }

    @Override
    public void performPreFlightChecks() {
        UsernameAndPasswordCredentials owner = getOwnerCredentials();
		SecurityTemplate.executeAs(owner.getUsername(), owner.getPassword(), new SecurityCallback<Object>() {
            @Override
            public Object doAs() {
                repositoryService.checkReferentialIntegrity(determineRepositoryChanges());
                return null;
            }
        });
    }

    private ChangeSet determineRepositoryChanges() {
        final DeployedApplication newDeployment = specification.getDeployedApplication();

        // Save the mappings that are stored as subnodes of the deployment that was just created
        final List<ConfigurationItem> createRoes = newArrayList();
        final List<ConfigurationItem> updateRoes = newArrayList();
        final List<String> deleteRoeIds = newArrayList();

        for (Delta operation : specification.getDeltas()) {
            switch (operation.getOperation()) {
                case CREATE:
                    createRoes.add(operation.getDeployed());
                    newDeployment.addDeployed(operation.getDeployed());
                    logger.info("The CI {} was {} in the repository", operation.getDeployed().getId(), operation.getOperation());
                    break;
                case DESTROY:
                    oldApp.getDeployeds().remove(operation.getPrevious());
                    updateRoes.add(oldApp);
                    deleteRoeIds.add(operation.getPrevious().getId());
                    logger.info("The CI {} was {} in the repository", operation.getPrevious().getId(), operation.getOperation());
                    break;
                case MODIFY:
	            case NOOP:
                    updateRoes.add(operation.getDeployed());
                    newDeployment.addDeployed(operation.getDeployed());
                    logger.info("The CI {} was {} in the repository", operation.getDeployed().getId(), operation.getOperation());
                    break;
            }
        }

        // Update the deployment to include the creating task id and to refer to the mappings that were just created
//        RepositoryObjectEntity updatedNewDeploymentEntity = context.toEntity(newDeployment);
//        updatedNewDeploymentEntity.setCreatingTaskId(getId());
        updateRoes.add(newDeployment);

        ChangeSet changeset = new ChangeSet();
        changeset.setDeleteCiIds(deleteRoeIds);
        changeset.setCreateCis(createRoes);
        changeset.setUpdateCis(updateRoes);

        return changeset;
    }

	@Override
	public void doAfterTaskStateChangedToDone() {
		UsernameAndPasswordCredentials owner = getOwnerCredentials();
		SecurityTemplate.executeAs(owner.getUsername(), owner.getPassword(), new SecurityCallback<Object>() {
			@Override
			public Object doAs() {
                repositoryService.execute(determineRepositoryChanges());
				return null;
			}
		});
	}

    private static final Logger logger = LoggerFactory.getLogger(UpgradeDeploymentTask.class);

    @Override
    public DeploymentType getDeploymentType() {
        return DeploymentTask.DeploymentType.UPGRADE;
    }
}
